from hotsos.core.plugintools import summary_entry_offset as idx
from hotsos.core.plugins.openstack.common import OpenstackChecksBase
from hotsos.core.plugins.openstack.neutron import NeutronHAInfo


class OpenstackSummary(OpenstackChecksBase):

    @idx(0)
    def __summary_release(self):
        return {'name': self.release_name,
                'days-to-eol': self.days_to_eol}

    @idx(1)
    def __summary_services(self):
        """Get string info for running services."""
        if self.systemd.services:
            return self.systemd.summary

    @idx(2)
    def __summary_dpkg(self):
        # require at least one core package to be installed to include
        # this in the report.
        if self.apt.core:
            return self.apt.all_formatted

    @idx(3)
    def __summary_docker_images(self):
        # require at least one core image to be in-use to include
        # this in the report.
        if self.docker.core:
            return self.docker.all_formatted

    @idx(4)
    def __summary_neutron_l3ha(self):
        routers = {}
        ha_info = NeutronHAInfo()
        for router in ha_info.ha_routers:
            state = router.ha_state
            if state in routers:
                routers[state].append(router.uuid)
            else:
                routers[state] = [router.uuid]

        if routers:
            return routers



import re

from hotsos.core.plugintools import summary_entry_offset as idx
from hotsos.core.plugins.kernel.common import KernelChecksBase
from hotsos.core.plugins.kernel.config import SystemdConfig
from hotsos.core.plugins.kernel.sysfs import CPU
from hotsos.core.plugins.kernel.memory import MemoryChecks


class KernelSummary(KernelChecksBase):

    @property
    def cpu_info(self):
        cpu = CPU()
        info = {}
        if cpu.vendor:
            info['vendor'] = cpu.vendor

        if cpu.model:
            model = cpu.model.lower()
            # strip intel or amd trailing info
            if 'intel' in cpu.vendor:
                ret = re.search(r'(.+) cpu @ .+', model)
            elif 'amd' in cpu.vendor:
                ret = re.search(r'(.+) \d+-core .+', model)

            if ret:
                model = ret.group(1).strip()

            info['model'] = model

        if cpu.smt is not None:
            if cpu.smt:
                info['smt'] = 'enabled'
            else:
                info['smt'] = 'disabled'

        if cpu.isolated is not None and cpu.isolated != '':
            info['isolated'] = cpu.isolated

        cpu_gov_all = cpu.cpufreq_scaling_governor_all
        info['cpufreq-scaling-governor'] = cpu_gov_all
        return info

    @idx(0)
    def __summary_version(self):
        if self.version:
            return self.version

    @idx(1)
    def __summary_boot(self):
        if self.boot_parameters:
            return ' '.join(self.boot_parameters)

    @idx(2)
    def __summary_systemd(self):
        cfg = SystemdConfig()
        if cfg.exists:
            if cfg.get('CPUAffinity'):
                return {'CPUAffinity': cfg.get('CPUAffinity')}

    @idx(3)
    def __summary_cpu(self):
        cpu_info = self.cpu_info
        if cpu_info:
            return cpu_info

    @idx(4)
    def __summary_memory(self):
        nodes = MemoryChecks().nodes_with_limited_high_order_memory_full
        if nodes:
            return nodes



from hotsos.core.config import HotSOSConfig
from hotsos.core.issues import IssuesManager, HotSOSScenariosWarning
from hotsos.core.log import log
from hotsos.core.ycheck.engine import (
    YDefsLoader,
    YDefsSection,
    YHandlerBase,
)


class Scenario(object):
    def __init__(self, name, checks, conclusions):
        log.debug("scenario: %s", name)
        self.name = name
        self._checks = checks
        self._conclusions = conclusions

    @property
    def checks(self):
        return {c.name: c for c in self._checks}

    @property
    def conclusions(self):
        return {c.name: c for c in self._conclusions}


class YScenarioChecker(YHandlerBase):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._scenarios = []

    def load(self):
        plugin_content = YDefsLoader('scenarios').plugin_defs
        if not plugin_content:
            return

        yscenarios = YDefsSection(HotSOSConfig.plugin_name, plugin_content)
        if (not HotSOSConfig.force_mode and yscenarios.requires and not
                yscenarios.requires.passes):
            log.debug("plugin '%s' scenarios pre-requisites not met - "
                      "skipping", HotSOSConfig.plugin_name)
            return

        log.debug("sections=%s, scenarios=%s",
                  len(yscenarios.branch_sections),
                  len(yscenarios.leaf_sections))

        to_skip = set()
        for scenario in yscenarios.leaf_sections:
            # Only register scenarios if requirements are satisfied.
            group_name = scenario.parent.name
            if (not HotSOSConfig.force_mode and
                    (group_name in to_skip or
                        (scenario.requires and not scenario.requires.passes))):
                log.debug("%s requirements not met - skipping scenario %s",
                          group_name, scenario.name)
                to_skip.add(group_name)
                continue

            scenario.checks.initialise(scenario.vars, scenario.input)
            scenario.conclusions.initialise(scenario.vars)
            self._scenarios.append(Scenario(scenario.name,
                                            scenario.checks,
                                            scenario.conclusions))

    @property
    def scenarios(self):
        return self._scenarios

    def _run_scenario_conclusion(self, scenario, issue_mgr):
        """ Determine the conclusion of this scenario. """
        results = {}
        # run all conclusions and use highest priority result(s). One or
        # more conclusions may share the same priority. All conclusions
        # that match and share the same priority will be used.
        for name, conc in scenario.conclusions.items():
            if conc.reached(scenario.checks):
                if conc.priority:
                    priority = conc.priority.value
                else:
                    priority = 1

                if priority in results:
                    results[priority].append(conc)
                else:
                    results[priority] = [conc]

                log.debug("conclusion reached: %s (priority=%s)", name,
                          priority)

        if results:
            highest = max(results.keys())
            log.debug("selecting highest priority=%s conclusions (%s)",
                      highest, len(results[highest]))
            for conc in results[highest]:
                issue_mgr.add(conc.issue, context=conc.issue_context)
        else:
            log.debug("no conclusions reached")

    def run(self):
        failed_scenarios = []
        issue_mgr = IssuesManager()
        for scenario in self.scenarios:
            log.debug("running scenario: %s", scenario.name)
            # catch failed scenarios and allow others to run
            try:
                self._run_scenario_conclusion(scenario, issue_mgr)
            except Exception:
                log.exception("caught exception when running scenario %s:",
                              scenario.name)
                failed_scenarios.append(scenario.name)

        if failed_scenarios:
            msg = ("One or more scenarios failed to run ({}) - run hotsos in "
                   "debug mode (--debug) to get more detail".
                   format(', '.join(failed_scenarios)))
            issue_mgr.add(HotSOSScenariosWarning(msg))



from hotsos.core.log import log
from cryptography import x509
from cryptography.hazmat.backends import default_backend

from datetime import datetime


class SSLCertificate(object):
    """
    This class instantiates an SSLCertificate object.
    """

    def __init__(self, certificate_path):
        """
        If the file in the path fails to read, it raises an exception.
        @param certificate_path: path to the certificate that we want to load
        in order to gather information from.
        """
        self.certificate_path = certificate_path
        try:
            with open(self.certificate_path, "rb") as fd:
                self.certificate = fd.read()
        except OSError as e:
            log.warning("Unable to read SSL certificate file %s, error: %s",
                        self.certificate_path, e)
            raise

    @property
    def certificate_expire_date(self):
        """
        Return datetime() of when the certificate expires
        """

        cert = x509.load_pem_x509_certificate(self.certificate,
                                              default_backend())
        return cert.not_valid_after

    @property
    def certificate_days_to_expire(self):
        """
        Return int(days) remaining until the certificate expires
        """
        today = datetime.today()
        certificate_date = self.certificate_expire_date
        days_to_expire = certificate_date - today
        return int(days_to_expire.days)

    @property
    def path(self):
        """
        Return str(path) where the certificate file is located
        """
        return str(self.certificate_path)


class SSLCertificatesHelper(object):

    def __init__(self, certificate, expire_days):
        """
        @param certificate: Expects an SSLCertificate object.
        @param expire_days: Expect an int(expire_days), it is used to check the
        days that remain until certificate expiration.
        """
        self.certificate = certificate
        self.expire_days = expire_days

    @property
    def certificate_expires_soon(self):
        """
        Returns True if certificate expires in less than int(self.expire_days)
        """
        return self.certificate.certificate_days_to_expire <= self.expire_days



import abc

from searchkit.utils import MPCache

from hotsos.core.config import HotSOSConfig


class HostHelpersBase(abc.ABC):

    def __init__(self, *args, **kwargs):
        self.cache = MPCache(self.cache_name,
                             'host_helpers_{}'.format(self.cache_type),
                             self.cache_root)
        super().__init__(*args, **kwargs)

    @property
    def cache_root(self):
        """
        By default caches are global to all plugins but this can be overridden
        if we want otherwise.
        """
        return HotSOSConfig.global_tmp_dir

    @abc.abstractproperty
    def cache_name(self):
        """ Unique name for cache used by instance of this object. """

    @abc.abstractproperty
    def cache_type(self):
        """
        Unique name for the type of cache used by instance of this object.
        """

    @abc.abstractmethod
    def cache_load(self):
        """ Load cache contents. """

    @abc.abstractmethod
    def cache_save(self):
        """ Save contents to cache. """
